@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
-# Copyright (C) 2010-2012 by Dick Kniep <dick.kniep@lindix.nl>
+# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2013 by Dick Kniep <dick.kniep@lindix.nl>
 #
 # PyHoca GUI is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -30,7 +30,7 @@ try:
     wxversion.select('2.8')
 except: pass
 
-# Python X2go
+# Python X2Go
 import x2go
 
 # gevent
@@ -104,7 +104,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
             self.sshProxyPasswordLbl = wx.StaticText(self, wx.ID_ANY, _(u'Password')+':', size=(-1, -1)) 
             self.sshProxyPasswordTxt = wx.TextCtrl(self, wx.ID_ANY, '', style=wx.TE_PROCESS_ENTER|wx.TE_PASSWORD, size=(120, -1)) 
             self.sshProxyPasswordTxt.SetFocus()
-            self.sshProxyLoginBtn = wx.Button(self, wx.ID_OK, _(u'  Start SSH tunnel  '))
+            self.sshProxyLoginBtn = wx.Button(self, wx.ID_OK, '  '+_(u'Start SSH tunnel')+'  ')
             self.sshProxyLoginBtn.SetDefault()
 
             _tab_order.extend([self.sshProxyUserTxt, self.sshProxyPasswordTxt, self.sshProxyLoginBtn, ])
@@ -281,18 +281,18 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
             sshproxy_force_password_auth = True
 
         wx.BeginBusyCursor()
-        session_uuid = self._PyHocaGUI._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
+        session_uuid = self._PyHocaGUI._X2GoClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
         try:
-            self._PyHocaGUI._X2goClient__connect_session(session_uuid, 
+            self._PyHocaGUI._X2GoClient__connect_session(session_uuid, 
                                                          username=username, 
                                                          password=password, 
                                                          force_password_auth=force_password_auth,
-                                                         add_to_known_hosts=False,
+                                                         add_to_known_hosts=self._PyHocaGUI.add_to_known_hosts,
                                                          sshproxy_user=sshproxy_user,
                                                          sshproxy_password=sshproxy_password,
                                                          sshproxy_force_password_auth=sshproxy_force_password_auth,
                                                         )
-            if not self._PyHocaGUI._X2goClient__server_valid_x2gouser(session_uuid):
+            if not self._PyHocaGUI._X2GoClient__server_valid_x2gouser(session_uuid):
                 self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
                                                  title=_(u'%s - connect failure') % self.current_profile_name,
                                                  text=_(u'User is not allowed to start X2Go sessions!'),
@@ -341,7 +341,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                                                  icon='auth_failed')
                 connect_failed = True
 
-        except x2go.X2goSSHProxyAuthenticationException:
+        except x2go.X2GoSSHProxyAuthenticationException:
             try: wx.EndBusyCursor()
             except: pass
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
@@ -378,7 +378,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                                              icon='auth_error')
             connect_failed = True
 
-        except x2go.X2goHostKeyException, e:
+        except x2go.X2GoHostKeyException, e:
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name, 
                                              title=_(u'%s - host key error') % self.current_profile_name, 
                                              text=_(u'The remote server\'s host key is invalid or has not been accepted by the user') + '!',
@@ -386,7 +386,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                                              timeout=4000)
             connect_failed = True
 
-        except x2go.X2goRemoteHomeException, e:
+        except x2go.X2GoRemoteHomeException, e:
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
                                              title=_(u'%s - missing home directory') % self.current_profile_name,
                                              text=_("The remote user's home directory does not exist."),
@@ -394,7 +394,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                                              timeout=4000)
             connect_failed = True
 
-        except x2go.X2goSSHProxyException, e:
+        except x2go.X2GoSSHProxyException, e:
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
                                              title=_(u'%s - key error') % self.current_profile_name,
                                              text='%s!' % str(e),
@@ -402,7 +402,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                                              timeout=4000)
             connect_failed = True
 
-        except x2go.X2goSessionException, e:
+        except x2go.X2GoSessionException, e:
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
                                              title=_(u'%s - auth error') % self.current_profile_name,
                                              text='%s!' % str(e),
@@ -411,11 +411,19 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
             connect_failed = True
 
         except x2go.SSHException, e:
+
+            if str(e).startswith('Host key for server ') and str(e).endswith(' does not match!'):
+                errmsg = _('Host key verification failed. The X2Go server may have been compromised.\n\nIt is also possible that the host key has just been changed.\n\nHowever, for security reasons the connection will not be established!!!')
+            else:
+                errmsg = str(e)
+
             self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
                                              title=_(u'%s - SSH error') % self.current_profile_name,
-                                             text='%s!' % str(e),
+                                             text='%s' % errmsg,
                                              icon='auth_error',
-                                             timeout=4000)
+                                             timeout=10000)
+
+
             connect_failed = True
 
         except:
@@ -437,7 +445,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
             self._PyHocaGUI.WakeUpIdle()
             self._PyHocaGUI.ExitMainLoop()
 
-        if self._PyHocaGUI._X2goClient__is_session_connected(session_uuid):
+        if self._PyHocaGUI._X2GoClient__is_session_connected(session_uuid):
             self._PyHocaGUI._post_authenticate(evt, session_uuid)
         self.sshproxy_started = False
         try: del self._PyHocaGUI._logon_windows[self.current_profile_name]
