@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
-# Copyright (C) 2010-2012 by Dick Kniep <dick.kniep@lindix.nl>
+# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2013 by Dick Kniep <dick.kniep@lindix.nl>
 #
 # PyHoca GUI is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -64,21 +64,26 @@ class PyHocaGUI_Menu_TaskbarManageProfile(wx.Menu):
 
         ID_EDITPROFILE = wx.NewId()
         ID_COPYPROFILE = wx.NewId()
+        ID_EXPORTPROFILE = wx.NewId()
         ID_DELETEPROFILE = wx.NewId()
 
         # preparing profile_name information for the main PyHocaGUI instance
         self._PyHocaGUI._eventid_profilenames_map[ID_EDITPROFILE] = \
             self._PyHocaGUI._eventid_profilenames_map[ID_COPYPROFILE] = \
+            self._PyHocaGUI._eventid_profilenames_map[ID_EXPORTPROFILE] = \
             self._PyHocaGUI._eventid_profilenames_map[ID_DELETEPROFILE] = profile_name
 
         self.Append(text=_(u"Edit Profile"), id=ID_EDITPROFILE)
         if not self._PyHocaGUI.args.single_session_profile:
             self.AppendSeparator()
             self.Append(text=_(u"Use as Template for New Profile"), id=ID_COPYPROFILE)
+            self.AppendSeparator()
+            self.Append(text=_(u"Export Profile"), id=ID_EXPORTPROFILE)
             self.Append(text=_(u"Delete Profile"), id=ID_DELETEPROFILE)
 
         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileEdit, id=ID_EDITPROFILE)
         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileCopy, id=ID_COPYPROFILE)
+        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileExport, id=ID_EXPORTPROFILE)
         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileDelete, id=ID_DELETEPROFILE)
 
 
@@ -103,8 +108,11 @@ class PyHocaGUI_Menu_TaskbarOptionsManager(wx.Menu):
         wx.Menu.__init__(self)
 
         ID_ABOUT = wx.NewId()
+        ID_ABOUT_PYTHONX2GO = wx.NewId()
         self.Append(id=ID_ABOUT, text=_(u"About %s (%s)...") % (self._PyHocaGUI.appname, self._PyHocaGUI.version))
+        self.Append(id=ID_ABOUT_PYTHONX2GO, text=_(u"About %s (%s)...") % ("Python X2Go", x2go.__VERSION__))
         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnAbout, id=ID_ABOUT)
+        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnAboutPythonX2Go, id=ID_ABOUT_PYTHONX2GO)
 
         if not self._PyHocaGUI.restricted_trayicon:
 
@@ -173,7 +181,7 @@ class PyHocaGUI_Menu_TaskbarSessionActions(wx.Menu):
         @type profile_name: C{str}
         @param session_name: X2Go session name this submenu is for
         @type session_name: C{str}
-        @param session_info: session info object (C{X2goServerSessionInfo*} from Python X2Go)
+        @param session_info: session info object (C{X2GoServerSessionInfo*} from Python X2Go)
         @type session_info: C{obj}
         @param status: status of this session (R for running, S for suspended)
         @type status: C{str}
@@ -212,7 +220,7 @@ class PyHocaGUI_Menu_TaskbarSessionActions(wx.Menu):
             self._PyHocaGUI._eventid_sessionnames_map[ID_SUSPENDSESSION] = \
             self._PyHocaGUI._eventid_sessionnames_map[ID_TERMINATESESSION] = session_name
 
-        _s = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True)
+        _s = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)
         _session_status = status
         if session_info is not None:
             _session_status = session_info.get_status()
@@ -248,22 +256,30 @@ class PyHocaGUI_Menu_TaskbarSessionActions(wx.Menu):
                     _ts = self.Append(text=_(u"Transfer Session (not possible)"), id=ID_TRANSFERSESSION_DISABLED)
                     _ts.Enable(False)
 
-            if self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session(): 
-                _ss = self.Append(text=_(u"Suspend Session (and disconnect/exit)"), id=ID_SUSPENDSESSION)
-            elif self._PyHocaGUI.disconnect_on_suspend and _s.has_terminal_session():
-                _ss = self.Append(text=_(u"Suspend Session (and disconnect)"), id=ID_SUSPENDSESSION)
+            if not _s.is_shadow_session():
+                if self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session(): 
+                    _ss = self.Append(text=_(u"Suspend Session (and disconnect/exit)"), id=ID_SUSPENDSESSION)
+                elif self._PyHocaGUI.disconnect_on_suspend and _s.has_terminal_session():
+                    _ss = self.Append(text=_(u"Suspend Session (and disconnect)"), id=ID_SUSPENDSESSION)
+                else:
+                    _ss = self.Append(text=_(u"Suspend Session"), id=ID_SUSPENDSESSION)
+
+                if _s.is_published_applications_provider() and not self._PyHocaGUI.get_profile_config(profile_name)['published']:
+                    _ss.Enable(False)
+        if not _s.is_shadow_session():
+            if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
+                self.Append(text=_(u"Terminate Session (and disconnect/exit)"), id=ID_SUSPENDSESSION)
+            elif self._PyHocaGUI.disconnect_on_terminate and _s.has_terminal_session():
+                self.Append(text=_(u"Terminate Session (and disconnect)"), id=ID_TERMINATESESSION)
             else:
-                _ss = self.Append(text=_(u"Suspend Session"), id=ID_SUSPENDSESSION)
-
-            if _s.is_published_applications_provider() and not self._PyHocaGUI.get_profile_config(profile_name)['published']:
-                _ss.Enable(False)
-
-        if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
-            self.Append(text=_(u"Terminate Session (and disconnect/exit)"), id=ID_SUSPENDSESSION)
-        elif self._PyHocaGUI.disconnect_on_terminate and _s.has_terminal_session():
-            self.Append(text=_(u"Terminate Session (and disconnect)"), id=ID_TERMINATESESSION)
+                self.Append(text=_(u"Terminate Session"), id=ID_TERMINATESESSION)
         else:
-            self.Append(text=_(u"Terminate Session"), id=ID_TERMINATESESSION)
+            if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
+                self.Append(text=_(u"End Desktop Sharing (and disconnect/exit)"), id=ID_SUSPENDSESSION)
+            elif self._PyHocaGUI.disconnect_on_terminate and _s.has_terminal_session():
+                self.Append(text=_(u"End Desktop Sharing (and disconnect)"), id=ID_TERMINATESESSION)
+            else:
+                self.Append(text=_(u"End Desktop Sharing"), id=ID_TERMINATESESSION)
 
         if _s is not None and _s.is_published_applications_provider() and self._PyHocaGUI.get_profile_config(profile_name)['published']:
             self.AppendSeparator()
@@ -322,15 +338,11 @@ class PyHocaGUI_Menu_TaskbarProfileSharedFolders(wx.Menu):
         self.Append(id=ID_SHARECUSTOMLOCALFOLDER, text=_(u"&Share custom local folder"))
 
         self.AppendSeparator()
+        self._PyHocaGUI._eventid_unshared_folders_map={}
 
         _exported_folders = self._PyHocaGUI.get_profile_config(profile_name, 'export')
-        _shared_folders = self._PyHocaGUI._X2goClient__profile_get_shared_folders(profile_name=profile_name, check_list_mounts=True) or []
-
-        _sharable_folders = []
-        for _folder_path in _exported_folders.keys():
-            _sharable_folders.append(_folder_path)
-
-        self._PyHocaGUI._eventid_unshared_folders_map={}
+        _shared_folders = self._PyHocaGUI._X2GoClient__profile_get_shared_folders(profile_name=profile_name, check_list_mounts=True) or []
+        _sharable_folders = _exported_folders.keys()
         _unshared_folders = [ f for f in _sharable_folders if f and f not in _shared_folders ]
 
         if _unshared_folders:
@@ -358,6 +370,8 @@ class PyHocaGUI_Menu_TaskbarProfileSharedFolders(wx.Menu):
         self.AppendSeparator()
 
         _remember_shared_folders_item = self.AppendCheckItem(id=ID_REMEMBERSHAREDFOLDERS, text=_(u"Restore shares in next session"))
+        if not self._PyHocaGUI._remember_shared_folders.has_key(profile_name):
+            self._PyHocaGUI._remember_shared_folders[profile_name] = self._PyHocaGUI.get_profile_config(profile_name, 'restoreexports')
         _remember_shared_folders_item.Check(self._PyHocaGUI._remember_shared_folders[profile_name])
 
         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShareCustomLocalFolder, id=ID_SHARECUSTOMLOCALFOLDER)
@@ -423,7 +437,7 @@ def _generate_Menu_PublishedApplications(_PyHocaGUI, caller=None, profile_name=N
 
     """
     _lang = _PyHocaGUI.lang
-    _pubapp_session = _PyHocaGUI.get_session_of_session_name(session_name, return_object=True)
+    _pubapp_session = _PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)
     try:
         menu_map = _pubapp_session.get_published_applications(lang=_lang, max_no_submenus=_PyHocaGUI.args.published_applications_no_submenus)
     except AttributeError:
@@ -509,6 +523,7 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
         ID_CONNECT=wx.NewId()
         ID_PUBAPPSESSIONSTART=wx.NewId()
         ID_SESSIONSTART=wx.NewId()
+        ID_SHADOWSESSIONSTART=wx.NewId()
         ID_LAUNCHAPPLICATION = wx.NewId()
         ID_CLEANSESSIONS = wx.NewId()
         ID_EDITPROFILEWHILECONNECTED = wx.NewId()
@@ -535,11 +550,17 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
 
             else:
 
-                self._PyHocaGUI._eventid_profilenames_map[ID_SESSIONSTART] = profile_name
+                self._PyHocaGUI._eventid_profilenames_map[ID_SESSIONSTART] = \
+                    self._PyHocaGUI._eventid_profilenames_map[ID_SHADOWSESSIONSTART] = profile_name
 
                 if current_profile_config['command'] in x2go.defaults.X2GO_DESKTOPSESSIONS:
                     self.Append(id=ID_SESSIONSTART, text='%s (%s)' % (_(u"Start &new Desktop Session"), current_profile_config['command']))
                     self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)
+
+                elif current_profile_config['command'] == 'SHADOW':
+                    start_shadow_session = self.Append(id=ID_SHADOWSESSIONSTART, text=_(u"Start Desktop Sharing Session"))
+                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShadowSessionStart, id=ID_SHADOWSESSIONSTART)
+
                 elif current_profile_config['command'] == '' and current_profile_config['published']:
                     _pub_app_start_item = None
                     if profile_name in self._PyHocaGUI._temp_launching_pubapp_profiles:
@@ -549,6 +570,9 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
                         self._PyHocaGUI._eventid_profilenames_map[ID_PUBAPPSESSIONSTART] = profile_name
                         _pub_app_start_item = self.Append(id=ID_PUBAPPSESSIONSTART, text=_(u"Retrieve Application Menu"))
                         self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPubAppSessionStart, id=ID_PUBAPPSESSIONSTART)
+                elif current_profile_config['command'] == 'RDP':
+                    self.Append(id=ID_SESSIONSTART, text=_(u"Start &new RDP Session"))
+                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)
                 else:
                     self.Append(id=ID_SESSIONSTART, text=_(u"Start &new Session"))
                     self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)
@@ -628,11 +652,14 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
                     self._PyHocaGUI._eventid_profilenames_map[ID_LAUNCHAPPLICATION] = \
                         self._PyHocaGUI._eventid_profilenames_map[ID_CLEANSESSIONS] = profile_name
 
-                    if current_profile_config['applications'] and current_profile_config['command'] in x2go.defaults.X2GO_DESKTOPSESSIONS and not current_profile_config['published']:
+                    if current_profile_config['applications'] and current_profile_config['command'] in x2go.defaults.X2GO_DESKTOPSESSIONS.keys() and not current_profile_config['published']:
                         self.AppendSeparator()
                         self.AppendMenu(id=ID_LAUNCHAPPLICATION, text=_(u"Launch Single Application"),
                                         submenu=PyHocaGUI_Menu_LaunchSingleApplication(self._PyHocaGUI, caller=self, profile_name=profile_name)
                                        )
+                        if current_profile_config['command'] != 'SHADOW' and not self._PyHocaGUI.restricted_trayicon:
+                            self.Append(id=ID_SHADOWSESSIONSTART, text=_(u"Start Desktop Sharing Session"))
+                            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShadowSessionStart, id=ID_SHADOWSESSIONSTART)
 
                     if current_profile_config['published']:
 
@@ -702,38 +729,50 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
 
                     if _session_list_names:
 
-                        self.AppendSeparator()
-
+                        _session_list_matching_profile = []
                         for session_name in _session_list_names:
 
-                            session = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True)
+                            session = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)
 
-                            _s_id = wx.NewId()
+                            if not session:
+                                continue
+                            else:
+                                _session_list_matching_profile.append(session)
 
-                            if _session_list[session_name].get_status() == 'R':
-                                state = _(u'Running')
-                            elif _session_list[session_name].get_status() == 'S':
-                                state = _(u'Suspended')
-                            _marker = ''
-                            if session and session.is_master_session():
-                                _marker = '(*)'
-                            if session:
-                                session_submenu = self.AppendMenu(id=_s_id, text=u'%s: »%s« %s' % (state, session_name, _marker),
-                                                                  submenu=PyHocaGUI_Menu_TaskbarSessionActions(self._PyHocaGUI, caller=self,
-                                                                                                               profile_name=profile_name,
-                                                                                                               session_name=session_name,
-                                                                                                               session_info=_session_list[session_name],
-                                                                                                              )
+                        if _session_list_matching_profile:
+
+                            self.AppendSeparator()
+
+                            for session in _session_list_matching_profile:
+
+                                session_name = session.get_session_name()
+
+                                _s_id = wx.NewId()
+
+                                if _session_list[session_name].get_status() == 'R':
+                                    state = _(u'Running')
+                                elif _session_list[session_name].get_status() == 'S':
+                                    state = _(u'Suspended')
+                                _marker = ''
+                                if session and session.is_master_session():
+                                    _marker = '(*)'
+                                if session:
+                                    session_submenu = self.AppendMenu(id=_s_id, text=u'%s: »%s« %s' % (state, session_name, _marker),
+                                                                      submenu=PyHocaGUI_Menu_TaskbarSessionActions(self._PyHocaGUI, caller=self,
+                                                                                                                   profile_name=profile_name,
+                                                                                                                   session_name=session_name,
+                                                                                                                   session_info=_session_list[session_name],
+                                                                                                                  )
                                                                  )
-                                if self._PyHocaGUI._temp_disabled_session_names.has_key(profile_name) and session_name in self._PyHocaGUI._temp_disabled_session_names[profile_name]:
-                                    session_submenu.Enable(False)
+                                    if self._PyHocaGUI._temp_disabled_session_names.has_key(profile_name) and session_name in self._PyHocaGUI._temp_disabled_session_names[profile_name]:
+                                        session_submenu.Enable(False)
 
-                        # redefine list of session names to decide if the clean all session menu item is not be shown
-                        _session_list_names = [ _s_name for _s_name in _session_list.keys() if not _session_list[_s_name].is_published_applications_provider() ]
+                            # redefine list of session names to decide if the clean all session menu item is not be shown
+                            _session_list_names = [ _s.get_session_name() for _s in _session_list_matching_profile if not _session_list[_s.get_session_name()].is_published_applications_provider() ]
 
-                        if _session_list_names:
-                            self.Append(id=ID_CLEANSESSIONS, text=_(u"&Clean all sessions"))
-                            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnCleanSessions, id=ID_CLEANSESSIONS)
+                            if _session_list_names:
+                                self.Append(id=ID_CLEANSESSIONS, text=_(u"&Clean all sessions"))
+                                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnCleanSessions, id=ID_CLEANSESSIONS)
 
                 if not self._PyHocaGUI.restricted_trayicon:
 
@@ -746,7 +785,7 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
                     self._PyHocaGUI._eventid_profilenames_map[ID_UNSHAREFOLDERS] = profile_name
 
                 if current_profile_config['useexports'] and \
-                   self._PyHocaGUI._X2goClient__profile_is_folder_sharing_available(profile_name=profile_name) and \
+                   self._PyHocaGUI._X2GoClient__profile_is_folder_sharing_available(profile_name=profile_name) and \
                    self._PyHocaGUI.get_master_session(profile_name) is not None and \
                    self._PyHocaGUI.is_session_name_enabled(profile_name, self._PyHocaGUI.get_master_session(profile_name).get_session_name()):
 
@@ -761,6 +800,13 @@ class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
                         _shared_folders.Enable(False)
 
 
+        if profile_name in self._PyHocaGUI.client_connected_profiles(return_profile_names=True) and not self._PyHocaGUI.restricted_trayicon:
+            self.AppendSeparator()
+            ID_SERVERINFO = wx.NewId()
+            self._PyHocaGUI._eventid_profilenames_map[ID_SERVERINFO] = profile_name
+            self.Append(id=ID_SERVERINFO, text=_(u"Server Information"))
+            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnViewServerInformation, id=ID_SERVERINFO)
+
         if profile_name in self._PyHocaGUI.client_connected_profiles(return_profile_names=True) and not self._PyHocaGUI.exit_on_disconnect:
             self.AppendSeparator()
             ID_DISCONNECT = wx.NewId()
@@ -874,8 +920,8 @@ class PyHocaGUI_Menu_TaskbarProfileNames(wx.Menu):
                                                                                 group_name=profile_group,
                                                                                 parent_group=_parent_group,
                                                                                 group_menus=True)
-                                    )
 
+                                    )
         if filter_profiles:
             _profile_names = [ p for p in _profile_names if p not in filter_profiles ]
 
@@ -909,6 +955,19 @@ class PyHocaGUI_Menu_TaskbarProfileNames(wx.Menu):
             _dummy = self.Append(text=_(u'No session profiles defined'), id=wx.NewId())
             _dummy.Enable(False)
 
+        else:
+            if bind_method is None:
+                self.AppendSeparator()
+                _export_id = wx.NewId()
+                _export_group_name = "%s/%s" % (parent_group, group_name)
+                _export_group_name = _export_group_name.strip("/")
+                self._PyHocaGUI._eventid_exportprofiles_map[_export_id] = _export_group_name
+                if not group_name:
+                    self.Append(text=_(u'Export all Profiles'), id=_export_id)
+                else:
+                    self.Append(text=_(u'Export Profile Group'), id=_export_id)
+                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileExport, id=_export_id)
+
     def OnUpdateUI(self, evt):
         profile_name = self._PyHocaGUI._eventid_profilenames_map[evt.GetId()]
         if profile_name in self._PyHocaGUI._temp_disabled_profile_names:
@@ -942,7 +1001,7 @@ class PyHocaGUI_Menu_TaskbarSessionManager(wx.Menu):
         ID_AUTHENTICATE = wx.NewId()
         ID_EXIT = wx.NewId()
 
-        _auth_menu_text = self._PyHocaGUI.start_on_connect and _(u'Start/Resume Session') or _(u'Connect Server')
+        _auth_menu_text = _(u'Connect Server')
         self.AppendMenu(id=ID_AUTHENTICATE, 
                         text=_auth_menu_text, 
                         submenu=PyHocaGUI_Menu_TaskbarProfileNames(self._PyHocaGUI, 
@@ -957,7 +1016,7 @@ class PyHocaGUI_Menu_TaskbarSessionManager(wx.Menu):
 
         _connected_sessions = False
         for profile_name in _profile_names:
-            if profile_name in self._PyHocaGUI._X2goClient__client_connected_sessions(return_profile_names=True):
+            if profile_name in self._PyHocaGUI._X2GoClient__client_connected_sessions(return_profile_names=True):
                 _connected_sessions = True
                 _this_id = wx.NewId()
 
@@ -966,7 +1025,7 @@ class PyHocaGUI_Menu_TaskbarSessionManager(wx.Menu):
                     self.AppendMenu(text=_menu_profile_name,
                                     id=_this_id,
                                     submenu=PyHocaGUI_Menu_TaskbarSessionProfile(self._PyHocaGUI, caller=self, profile_name=profile_name))
-                except x2go.x2go_exceptions.X2goSessionRegistryException:
+                except x2go.x2go_exceptions.X2GoSessionRegistryException:
                     pass
 
         if _connected_sessions:
