@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
-# Copyright (C) 2010-2012 by Dick Kniep <dick.kniep@lindix.nl>
+# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2013 by Dick Kniep <dick.kniep@lindix.nl>
 #
 # PyHoca GUI is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -30,7 +30,7 @@ try:
     wxversion.select('2.8')
 except: pass
 
-# Python X2go
+# Python X2Go
 import x2go
 
 # gevent
@@ -144,7 +144,7 @@ class PyHocaGUI_DialogBoxSessionTitle(wx.Dialog):
         """
         title = self.titleTxt.GetValue()
 
-        _session = self._PyHocaGUI._X2goClient__get_session_of_session_name(session_name=self.current_session_name, return_object=True)
+        _session = self._PyHocaGUI._X2GoClient__get_session_of_session_name(session_name=self.current_session_name, return_object=True)
         _session.set_session_window_title(title=title)
 
         self.Close()
